/// This is probably just a fancy Cell or something.

pub enum Task <S, R>
{
    Stopped (S),
    Running (R),
}

#[derive (Debug, thiserror::Error)]
pub enum Error
{
    #[error ("tried to start already-running task")]
    AlreadyRunning,
    #[error ("tried to stop already-stopped task")]
    AlreadyStopped,
    #[error ("cannot access a task's inner object while it's running")]
    CantAccessWhileRunning,
}

impl <S, R> From <S> for Task <S, R>
{
    fn from (x: S) -> Self
    {
        Self::Stopped (x)
    }
}

impl <S, R> Task <S, R>
{
    pub fn is_running (&self) -> bool
    {
        match self
        {
            Self::Running (_) => true,
            Self::Stopped (_) => false,
        }
    }
    
    pub fn try_inner (&self) -> Result <&S, Error>
    {
        match self
        {
            Self::Running (_) => Err (Error::CantAccessWhileRunning),
            Self::Stopped (x) => Ok (x),
        }
    }
    
    pub fn try_inner_mut (&mut self) -> Result <&mut S, Error>
    {
        match self
        {
            Self::Running (_) => Err (Error::CantAccessWhileRunning),
            Self::Stopped (x) => Ok (x),
        }
    }
    
    pub fn start (&mut self, x: R) -> Result <S, Error>
    {
        match std::mem::replace (self, Self::Running (x))
        {
            Self::Running (_) => Err (Error::AlreadyRunning),
            Self::Stopped (x) => Ok (x),
        }
    }
    
    pub fn stop (&mut self, x: S) -> Result <R, Error>
    {
        match std::mem::replace (self, Self::Stopped (x))
        {
            Self::Stopped (_) => Err (Error::AlreadyStopped),
            Self::Running (x) => Ok (x),
        }
    }
}
